import { world, system } from "@minecraft/server";

world.afterEvents.worldInitialize.subscribe(() => {
    const players = world.getPlayers();
    players.forEach(player => {
        if (!player.hasDynamicProperty("cooldown")) {
            player.setDynamicProperty("cooldown", 0);
        }
    });
});

const cooldownTimes = {
    "tunekeshi:b3": 3,
    "tunekeshi:b4": 3,
    "tunekeshi:b6": 3,
    "tunekeshi:b7": 1,
    "tunekeshi:b8": 3,
    "tunekeshi:b9": 15,
    "tunekeshi:b10": 7,
    "tunekeshi:b11": 10,
    "tunekeshi:b12": 3,
    "tunekeshi:b14": 4,
    "tunekeshi:b16": 15,
    "tunekeshi:b17": 5,
    "tunekeshi:b18": 3,
    "tunekeshi:b19": 3,
    "tunekeshi:b20": 3,
    "tunekeshi:b21": 3,
    "tunekeshi:b22": 3,
    "tunekeshi:b25": 3,
    "tunekeshi:b26": 3,
    "tunekeshi:b27": 3,
    "tunekeshi:b28": 3,
};

system.runInterval(() => {
    const players = world.getPlayers();
    players.forEach(player => {
        let cooldown = player.getDynamicProperty("cooldown") || 0;
        if (cooldown > 0) {
            cooldown = Math.max(cooldown - 1, 0);
            player.setDynamicProperty("cooldown", cooldown);
        }
    });
}, 20);

world.beforeEvents.itemUse.subscribe(event => {
    const { itemStack, source } = event;
    if (!source || !itemStack) return;

    let cooldown = source.getDynamicProperty("cooldown") || 0;

    if (cooldown > 0) {
        event.cancel = true;
        return;
    }

    const itemId = itemStack.typeId;
    const cooldownTime = cooldownTimes[itemId];
    
    if (cooldownTime !== undefined) {
        source.setDynamicProperty("cooldown", cooldownTime);
        executeItemAction(source, itemId);
    }
});

function executeItemAction(source, itemId) {
    const head = source.getHeadLocation();
    const view = source.getViewDirection();

    switch (itemId) {

        case "tunekeshi:b3":
            source.runCommandAsync("summon tunekeshi:sa_b3 ^ ^1.8 ^1.5");
            break;
        
        case "tunekeshi:b4":
            spawnProjectile(source, "tunekeshi:sa_b4", head, view);
            break;
        
        case "tunekeshi:b6":
            source.runCommandAsync("execute as @e[r=3,type=!player] at @s run summon lightning_bolt");
            source.runCommandAsync("execute as @e[r=3,type=!player] at @s run summon tunekeshi:ice");
            break;

        case "tunekeshi:b7":
            source.runCommandAsync("summon tunekeshi:sa_b7 ^ ^3.8 ^1.5");
            source.runCommandAsync("summon tunekeshi:sa_b7 ^ ^2.8 ^1.5");
            source.runCommandAsync("summon tunekeshi:sa_b7 ^ ^1.8 ^1.5");
            source.runCommandAsync("effect @s resistance 1 255 true");
            source.runCommandAsync("particle sa_b7 ^ ^2.8 ^2.5");
            break;
        
        case "tunekeshi:b8":
            source.runCommandAsync("summon tunekeshi:sa_b8");
            break;
        
        case "tunekeshi:b9":
            source.addTag("wither");
            break;

        case "tunekeshi:b10":
            source.runCommandAsync("execute as @e[r=10,type=!player] at @s run summon lightning_bolt");
            source.runCommandAsync("effect @s fire_resistance 1 255 true");
            break;

        case "tunekeshi:b11":
            source.runCommandAsync("execute as @e[r=10,type=!tunekeshi:ice,type=!player] at @s run summon tunekeshi:ice");
            break;

        case "tunekeshi:b12":
            spawnProjectile(source, "tunekeshi:sa_b12", head, view);
            break;
        
        case "tunekeshi:b14":
            source.runCommandAsync('scoreboard players add @s level1 7');
            break;
        
        case "tunekeshi:b16":
            source.runCommandAsync("summon tunekeshi:sa_b16 ^ ^2.8 ^1.5");
            break;

        case "tunekeshi:b17":
            spawnProjectile(source, "tunekeshi:sa_b17", head, view);
            break;

        case "tunekeshi:b18":
            spawnProjectile(source, "tunekeshi:sa_b18", head, view);
            break;

        case "tunekeshi:b19":
            source.runCommandAsync("summon tunekeshi:sa_b19 ^ ^2.8 ^1.5");
            break;

        case "tunekeshi:b20":
            source.runCommandAsync("summon tunekeshi:sa_b19 ^ ^2.8 ^1.5");
            break;

        case "tunekeshi:b21":
            spawnProjectile(source, "tunekeshi:sa_b21", head, view);
            break;

        case "tunekeshi:b22":
            spawnProjectile(source, "tunekeshi:sa_b22", head, view);
            break;

        case "tunekeshi:b25":
            spawnProjectile(source, "tunekeshi:sa_b18", head, view);
            break;

        case "tunekeshi:b26":
            spawnProjectile(source, "tunekeshi:sa_b18", head, view);
            break;

        case "tunekeshi:b27":
            source.applyKnockback(vx, vz, 3, vy * 0.1);
            source.runCommandAsync(`execute at @s run particle minecraft:huge_explosion_emitter ^^^-1`);
            source.runCommandAsync(`execute at @s run damage @e[r=4,type=!player,type=!item] 10`);
            source.runCommandAsync(`execute at @s run particle sa_b27 ^^^`);
            break;

        case "tunekeshi:b28":
            source.applyKnockback(vx, vz, 3, vy * 0.1);
            source.runCommandAsync(`execute at @s run particle minecraft:huge_explosion_emitter ^^^-1`);
            source.runCommandAsync(`execute at @s run damage @e[r=4,type=!player,type=!item] 7`);
            source.runCommandAsync(`execute at @s run particle sa_b28 ^^^`);
            source.runCommandAsync("execute as @e[r=4,type=!player,type=!tunekeshi:ice] at @s run summon tunekeshi:ice");
            break;
    }
};

function spawnProjectile(source, projectileId, head, view) {
    const spawnLocation = {
        x: head.x + view.x,
        y: head.y + view.y,
        z: head.z + view.z
    };
    const projectile = source.dimension.spawnEntity(projectileId, spawnLocation);
    const viewDirection = source.getViewDirection();
    projectile.applyImpulse({
        x: viewDirection.x * 2,
        y: viewDirection.y * 2,
        z: viewDirection.z * 2
    });
}